﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Diagnostics;
using System.Security;
using System.Security.Principal;
using System.Reflection;

using OpenLibSys;


namespace VirtualCheckerWpf
{
    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window
    {
        Ols ols;

        public MainWindow()
        {
            InitializeComponent();

            if (!IsAdmin())
            {
                RunAsRestart();
                Environment.Exit(0);
            }
            Init();
        }

        private bool IsAdmin()
        {
            OperatingSystem osInfo = Environment.OSVersion;
            if (osInfo.Platform == PlatformID.Win32Windows)
            {
                return true;
            }
            else
            {
                WindowsIdentity usrId = WindowsIdentity.GetCurrent();
                WindowsPrincipal p = new WindowsPrincipal(usrId);
                return p.IsInRole(WindowsBuiltInRole.Administrator);
            }
        }

        private bool RunAsRestart()
        {
            string[] args = Environment.GetCommandLineArgs();

            foreach (string s in args)
            {
                if (s.Equals("runas"))
                {
                    return false;
                }
            }
            ProcessStartInfo startInfo = new ProcessStartInfo();
            startInfo.UseShellExecute = true;
            startInfo.WorkingDirectory = Environment.CurrentDirectory;
            startInfo.FileName = Assembly.GetEntryAssembly().Location;
            startInfo.Verb = "runas";
            startInfo.Arguments = "runas";

            try
            {
                Process.Start(startInfo);
            }
            catch
            {
                return false;
            }
            return true;
        }

        private void Init()
        {
            try
            {
                //-----------------------------------------------------------------------------
                // 初期化
                //-----------------------------------------------------------------------------
                ols = new Ols();

                // Check support library sutatus
                switch (ols.GetStatus())
                {
                    case (uint)Ols.Status.NO_ERROR:
                        break;
                    case (uint)Ols.Status.DLL_NOT_FOUND:
                        MessageBox.Show("Status Error!! DLL_NOT_FOUND");
                        Environment.Exit(0);
                        break;
                    case (uint)Ols.Status.DLL_INCORRECT_VERSION:
                        MessageBox.Show("Status Error!! DLL_INCORRECT_VERSION");
                        Environment.Exit(0);
                        break;
                    case (uint)Ols.Status.DLL_INITIALIZE_ERROR:
                        MessageBox.Show("Status Error!! DLL_INITIALIZE_ERROR");
                        Environment.Exit(0);
                        break;
                }

                // Check WinRing0 status
                switch (ols.GetDllStatus())
                {
                    case (uint)Ols.OlsDllStatus.OLS_DLL_NO_ERROR:
                        break;
                    case (uint)Ols.OlsDllStatus.OLS_DLL_DRIVER_NOT_LOADED:
                        MessageBox.Show("DLL Status Error!! OLS_DRIVER_NOT_LOADED");
                        Environment.Exit(0);
                        break;
                    case (uint)Ols.OlsDllStatus.OLS_DLL_UNSUPPORTED_PLATFORM:
                        MessageBox.Show("DLL Status Error!! OLS_UNSUPPORTED_PLATFORM");
                        Environment.Exit(0);
                        break;
                    case (uint)Ols.OlsDllStatus.OLS_DLL_DRIVER_NOT_FOUND:
                        MessageBox.Show("DLL Status Error!! OLS_DLL_DRIVER_NOT_FOUND");
                        Environment.Exit(0);
                        break;
                    case (uint)Ols.OlsDllStatus.OLS_DLL_DRIVER_UNLOADED:
                        MessageBox.Show("DLL Status Error!! OLS_DLL_DRIVER_UNLOADED");
                        Environment.Exit(0);
                        break;
                    case (uint)Ols.OlsDllStatus.OLS_DLL_DRIVER_NOT_LOADED_ON_NETWORK:
                        MessageBox.Show("DLL Status Error!! DRIVER_NOT_LOADED_ON_NETWORK");
                        Environment.Exit(0);
                        break;
                    case (uint)Ols.OlsDllStatus.OLS_DLL_UNKNOWN_ERROR:
                        MessageBox.Show("DLL Status Error!! OLS_DLL_UNKNOWN_ERROR");
                        Environment.Exit(0);
                        break;
                }

                String str = "";

                if (IsIntelVtSupported())
                {
                    if (IsIntelVtEnabled())
                    {
                        str = "Intel VT 有効";
                    }
                    else
                    {
                        str = "Intel VT 無効";
                    }
                }
                else if (IsAmdvSupported())
                {
                    if (IsAmdvEnabled())
                    {
                        str = "AMD-V 有効";
                    }
                    else
                    {
                        str = "AMD-V 無効";
                    }
                }
                else
                {
                    str = "Intel VT/AMD-V 未対応";
                }

                label1.Content = str;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error");
                Environment.Exit(0);
            }
        }

        private bool IsIntelVtSupported()
        {
            uint eax = 0, ebx = 0, ecx = 0, edx = 0;
            if (ols.CpuidPx(0x1, ref eax, ref ebx, ref ecx, ref edx, (UIntPtr)1) != 0 && (ecx & (1 << 5)) != 0)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        private bool IsIntelVtEnabled()
        {
            uint eax = 0, edx = 0;
            if (ols.RdmsrPx(0x3A, ref eax, ref edx, (UIntPtr)1) != 0 && ((eax & (1 << 1)) != 0 || (eax & (1 << 2)) != 0))
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        private bool IsAmdvSupported()
        {
            uint eax = 0, ebx = 0, ecx = 0, edx = 0;
            ols.CpuidPx(0x80000000, ref eax, ref ebx, ref ecx, ref edx, (UIntPtr)1);
            if (eax < 0x80000001)
            {
                return false;
            }

            if (ols.CpuidPx(0x80000001, ref eax, ref ebx, ref ecx, ref edx, (UIntPtr)1) != 0 && (ecx & (1 << 2)) != 0)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        private bool IsAmdvEnabled()
        {
            uint eax = 0, edx = 0;
            if (ols.RdmsrPx(0xC0010114, ref eax, ref edx, (UIntPtr)1) != 0 && (eax & (1 << 4)) == 0)
            {
                return true;
            }
            else
            {
                return false;
            }
        }
    }
}